@@ -611,6 +611,14 @@ extract_value(HeapTuple tuple, Relation heap, Relation index)
 
 	tupdesc = RelationGetDescr(heap);
 	attnum = index->rd_index->indkey.values[0];
+
+	/* attnum could be 0 for expression indexes */
+	if (attnum == 0)
+	{
+		/* TODO: re-evaluate expression and construct an actual value */
+		return NULL;
+	}
+
 	value = heap_getattr(tuple, attnum, tupdesc, &isnull);
 
 	if (isnull)
@@ -1451,7 +1459,7 @@ SennaScanOpen(SennaScanDesc *desc,
 			{
 				/* %foo% is converted to "foo" */
 				if (buf.len > 0)
-					appendStringInfoChar(&buf, '+');
+					appendStringInfoString(&buf, " +");
 				appendStringInfoChar(&buf, '"');
 				appendBinaryStringInfo(&buf, str + 1, len - 2);
 				appendStringInfoChar(&buf, '"');
@@ -1498,7 +1506,7 @@ SennaScanOpen(SennaScanDesc *desc,
 						if (!quoted)
 						{
 							if (buf.len > 0)
-								appendStringInfoChar(&buf, '+');
+								appendStringInfoString(&buf, " +");
 							appendStringInfoChar(&buf, '"');
 							quoted = true;
 						}
@@ -1517,7 +1525,7 @@ SennaScanOpen(SennaScanDesc *desc,
 		else
 		{
 			if (buf.len > 0)
-				appendStringInfoChar(&buf, '+');
+				appendStringInfoString(&buf, " +");
 			/* just copy for senna query */
 			appendBinaryStringInfo(&buf, str, len);
 		}
